#define SerialMon Serial
#define SerialAT Serial1
#define TINY_GSM_MODEM_ESP8266
// Pin decleration
#define moisturePin A1
#define relayPin A2

#include <Adafruit_CircuitPlayground.h>
#include <FiniteStateMachine.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <ESP8266_Lib.h>
#include "Flasher.h"
#include <TinyGsmClient.h>

// Variables decleration
int moisture;
int dryThreshold = 1023;
int pumpingTime = 1500;       // Pumpung time (milliseconds)
int perculationTime = 10000;  // Time for water to perculate after pumping (seconds)
String mode = "WAITING";
String notificationStatus = "1";
bool irrigateToday = true;
Flasher flaser(0, 255, 0, 100);

// WiFi connection credentials
const char wifiSSID[] = "Your_WiFi_SSID";
const char wifiPass[] = "Your_WiFi_Password";
char blynkAuth[] = "Your_Blynk_Auth_Key";

// Network enviorment
ESP8266 wifi(&SerialAT);
TinyGsm modem(SerialAT);
TinyGsmClientSecure client(modem);

#include "Connections.h"

// Check the moisture of the soil
void onCheckStart() {
  
  moisture = analogRead(moisturePin);
  
}

// State machine states
State checkState = State(onCheckStart, checkUpdate, NULL);
State irrigateState = State(irrigateUpdate);
State notifyState = State(notifyUpdate);
State restState = State(restUpdate);

// State machine
FSM stateMachine = FSM(checkState);

// Setup function
void setup() {
  
    CircuitPlayground.begin();
    pinMode(relayPin, OUTPUT);
    digitalWrite(relayPin, HIGH);
  
    initModem();
    connectToWifi();

}

// Loop function
void loop() {

    stateMachine.update();
  
}

// Envoke actions according to soil moistureness
void checkUpdate() {
  
    if (irrigateToday) {

        if (moisture >= dryThreshold) {
            stateMachine.transitionTo(irrigateState);       
        }
        else if (mode == "PUMPING") {
            flaser.Destroy();
            stateMachine.transitionTo(notifyState);
        }
        
    }
    else {
        stateMachine.transitionTo(restState);
    }

    moisture = analogRead(moisturePin);
    
}

// Turn on the pump and let perculate
void irrigateUpdate() {
  
    mode = "PUMPING";
    pump();
    
    // Let to perculate
    unsigned long start = millis();
    while (millis() - start < perculationTime) {
        flaser.Update();
    }

    stateMachine.transitionTo(checkState);
    
}

// Notify the user that the plant has been irrigated
void notifyUpdate() {
  
    Blynk.begin(blynkAuth, wifi, wifiSSID, wifiPass);
    SerialMon.println("Send notification");
    Blynk.virtualWrite(V1, "IRRIGATED");
    mode = "WAITING";
    stateMachine.transitionTo(checkState);
    
}

// Rest until today will be a dry day
void restUpdate() {

    if (irrigateToday) {
        stateMachine.transitionTo(checkState);
    }
    
}

// Acticate the water pump for specific time
void pump() {

    unsigned long start = millis();
    digitalWrite(relayPin, LOW);
    while (millis() - start < pumpingTime) {
        flaser.Update();
    }
    digitalWrite(relayPin, HIGH);
    
}

// Get rain chances for today
BLYNK_WRITE(V0) {
  
    int rainChance = param.asInt();
    SerialMon.print("Rain Chance: ");
    SerialMon.println(rainChance);
    
    if (rainChance = 0) {
        irrigateToday = true;
    }
    else {
        irrigateToday = false;
    }
    
}
